@@ -8,11 +8,11 @@ The required format for a heuristic is to take in a circuit, a vector of paramet
 
 def greedy(circ, v, weight, options):
     """Defines a heuristic that results in greedy search, which focuses soley on minimizing the eval_func, and behaves somewhat similarly to depth first sarch."""
-    return options.eval_func(options.target, circ.matrix(v))
+    return options.objective.gen_eval_func(circ, options)(v)
 
 def astar(circ, v, weight, options):
     """Defines a heuristic that combines the weight of the circuit with the value from eval_func.  It generally gives similar quality results to djikstra, but with a drastic reduction in the number of node evaluations."""
-    return 10.0*options.eval_func(options.target, circ.matrix(v)) + weight
+    return 10.0*options.objective.gen_eval_func(circ, options)(v) + weight
 
 def djikstra(circ, v, weight, options):
     """Defines a heuristic that relies only on the weight, which gurantees a minimal-weight final solution, at the expense of a long runtime.  It behaves somewhat similarly to breadth first search."""
