@@ -66,7 +66,7 @@ class Options():
             return retval
         elif name in self.defaults:
             return self.defaults[name]
-        raise AttributeError("Could not find option {}".format(name))
+        raise AttributeError("Could not find option '{}'".format(name))
 
     def __setattr__(self, name, value):
         if name not in _options_actual_parameters:
@@ -85,6 +85,29 @@ class Options():
         else:
             return False
 
+    def manually_entered(self, *names, location="dict", operator="all"):
+        if location == "dict":
+            for name in names:
+                if name in self.__dict__:
+                    if operator == "any":
+                        return True
+                elif operator == "all":
+                    return False
+        elif location == "defaults":
+            for name in names:
+                if name in self.defaults:
+                    if operator == "any":
+                        return True
+                elif operator == "all":
+                    return False
+        elif location == "smart_defaults":
+            for name in names:
+                if name in self.smart_defaults:
+                    if operator == "any":
+                        return True
+                elif operator == "all":
+                    return False
+
     def empty_copy(self):
         """Create an Options object with the same defaults but without any specific values."""
         newOptions = Options(self.defaults)
@@ -114,18 +137,19 @@ class Options():
         newOptions = self.copy()
         if other is not None:
             newOptions._update_dict(other.__dict__)
-            newOptions.defaults.update(other.defaults)
-            newOptions.smart_defaults.update(other.smart_defaults)
+            newOptions.set_defaults(**other.defaults)
+            newOptions.set_smart_defaults(**other.smart_defaults)
             newOptions.required = self.required.union(other.required)
         newOptions._update_dict(xtraargs)
+        newOptions.cache = dict()
         return newOptions
 
     def update(self, other=None, **xtraargs):
         """Mutate the current Options object with the contents of other and xtraargs."""
         if other is not None:
             self._update_dict(other.__dict__)
-            self.defaults.update(other.defaults)
-            self.smart_defaults.update(other.smart_defaults)
+            self.set_defaults(**other.defaults)
+            self.set_smart_defaults(**other.smart_defaults)
             self.required.update(other.required)
         self._update_dict(xtraargs)
         self.cache = dict()
@@ -185,7 +209,10 @@ class Options():
     def generate_cache(self):
         """Caches valuesa for all functions in smart_defaults."""
         for key in self.smart_defaults:
-            getattr(self, key) 
+            try:
+                getattr(self, key) 
+            except:
+                pass
             # this may look weird, but calling getattr on these keys will populate
             # the cache with any keys that have not been already cached.
 
