@@ -23,6 +23,9 @@ except ImportError:
 
 from . import unitaries
 
+# this is for backwards compatability
+from .comparison import *
+
 def matrix_product(*LU):
     """Performs matrix multiplication of a list of matrices."""
     result = np.array(np.eye(LU[0].shape[0]), dtype='complex128')
@@ -41,98 +44,6 @@ def op_norm(A):
     """An implementation of the l1-l1 operator norm."""
     return max([np.linalg.norm(x,ord=1) for x in A])
 
-def matrix_distance_squared(A,B):
-    # this distance function is designed to be phase agnostic
-    # optimized implementation
-    return np.abs(1 - np.abs(np.sum(np.multiply(A,np.conj(B)))) / A.shape[0])
-    #original implementation
-    #return 1 - np.abs(np.trace(np.dot(A,B.T.conjugate()))) / A.shape[0]
-
-def matrix_distance_squared_jac(U, M, J):
-    S = np.sum(np.multiply(U, np.conj(M)))
-    dsq = 1 - np.abs(S)/U.shape[0]
-    JU = np.array([np.multiply(U,np.conj(K)) for K in J])
-    JUS = np.sum(JU, axis=(1,2))
-    jacs = -(np.real(S)*np.real(JUS) + np.imag(S)*np.imag(JUS))*U.shape[0] / np.abs(S)
-    return (dsq, jacs)
-
-def matrix_residuals(A, B, I):
-    M = np.dot(B,np.conj(A.T))
-    #M *= np.abs(M[0][0])/M[0][0]
-    Re, Im = np.real(M), np.imag(M)
-    Re -= I
-    Re = np.reshape(Re, (1,-1))
-    Im = np.reshape(Im, (1,-1))
-    return np.append(Re, Im)
-
-def matrix_residuals_jac(U, M, J):
-    Ut = np.conj(U.T)
-    #M2 = np.dot(M, Ut)
-    #Ut *= np.abs(M2[0][0])/M2[0][0]
-    JU = [np.matmul(K, Ut) for K in J]
-    JU = np.array([np.append(np.reshape(np.real(K), (1,-1)), np.reshape(np.imag(K), (1,-1))) for K in JU])
-    return JU.T
-
-def matrix_residuals_v2(A, B, I):
-    # TODO examine how this function behaves compared to the original implementation.  Faster?  Behaves better?  Roughly the same?  Exactly the same?
-    M = B - A
-    #M *= np.abs(M[0][0])/M[0][0]
-    Re, Im = np.real(M), np.imag(M)
-    Re = np.reshape(Re, (1,-1))
-    Im = np.reshape(Im, (1,-1))
-    return np.append(Re, Im)
-
-def matrix_residuals_v2_jac(U, M, J):
-    JU = np.array([np.append(np.reshape(np.real(K), (1,-1)), np.reshape(np.imag(K), (1,-1))) for K in J])
-    return JU.T
-
-def matrix_residuals_slice(slices, A, B, I):
-    M = (B - A)[slices]
-    #M *= np.abs(M[0][0])/M[0][0]
-    Re, Im = np.real(M), np.imag(M)
-    Re = np.reshape(Re, (1,-1))
-    Im = np.reshape(Im, (1,-1))
-    return np.append(Re, Im)
-
-def matrix_residuals_slice_jac(slices, A, B, J):
-    JU = np.array([np.append(np.reshape(np.real(K[slices]), (1,-1)), np.reshape(np.imag(K[slices]), (1,-1))) for K in J])
-    return JU.T
-
-def matrix_residuals_blacklist(badrows, badcols, A, B, I):
-    M = (B - A)
-    M = np.delete(M, badrows, 0)
-    M = np.delete(M, badcols, 1)
-    #M *= np.abs(M[0][0])/M[0][0]
-    Re, Im = np.real(M), np.imag(M)
-    Re = np.reshape(Re, (1,-1))
-    Im = np.reshape(Im, (1,-1))
-    return np.append(Re, Im)
-
-def distance_with_initial_state(stateA,stateB,A,B):
-    return np.abs(1-np.abs(np.vdot(A.dot(stateA),B.dot(stateB))))
-
-def distance_with_initial_state_jac(stateA,stateB,A,B,J):
-    si = A.dot(stateA)
-    s = np.vdot(si,B.dot(stateB))
-    dist = 1-np.abs(s)
-    vu = np.array([np.vdot(si, K.dot(stateB)) for K in J])
-    jacs = -(np.real(s)*np.real(vu) + np.imag(s)*np.imag(vu))/np.abs(s)
-    return (dist, jacs)
-
-def residuals_with_initial_state(stateA, stateB,A,B,I):
-    v = 1-np.conj(A.dot(stateA)) * B.dot(stateB)
-    Re, Im = np.real(v), np.imag(v)
-    return np.append(Re,Im)
-
-def residuals_with_initial_state_jac(stateA,stateB, U, M, J):
-    vc = np.conj(U.dot(stateA))
-    vu = [-vc * K.dot(stateB) for K in J]
-    vu = np.array([np.append(np.real(v),np.imag(v)) for v in vu])
-    return vu.T
-
-def matrix_residuals_blacklist_jac(slices, A, B, J):
-    JU = np.array([np.append(np.reshape(np.real(np.delete(np.delete(K, badrows, 0), badcols, 1)), (1,-1)), np.reshape(np.imag(np.delete(np.delete(K, badrows, 0), badcols, 1)), (1,-1))) for K in J])
-    return JU.T
 
 def nearest_unitary(A):
     """
@@ -188,9 +99,6 @@ def downgrade_qudits_residuals_jac(di, df, A, B, J):
     JU = np.array([np.append(np.reshape(np.real(K), (1,-1)), np.reshape(np.imag(K), (1,-1))) for K in JU])
     return JU.T
 
-def eval_func_from_residuals(f, A, B):
-    return np.sum(np.square(f(A,B,I=np.array(np.eye(A.shape[0]), dtype='float64'))))
-
 def generate_stateprep_target_matrix(state):
     # WARNING: the matrix generated by this function (currently) is not unitary, and therefore should only be used with functions like matrix_residuals_slice
     state = np.array(state, dtype='complex128')
@@ -198,9 +106,6 @@ def generate_stateprep_target_matrix(state):
     M[0] = state
     return M
 
-def matrix_distance(A,B):
-    return np.abs(np.sqrt(np.abs(matrix_distance_squared(A,B))))
-
 def re_rot_z(theta, old_z):
     old_z[0,0] = np.exp(-1j*theta/2)
     old_z[1,1] = np.exp(1j*theta/2)
